<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Controller\Action;

use Dedi\SyliusVoucherPlugin\Model\VoucherConfigurationInterface;
use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Dedi\SyliusVoucherPlugin\Provider\VoucherChannelConfigurationProviderInterface;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Dedi\SyliusVoucherPlugin\Security\VoucherVoter;
use Knp\Bundle\SnappyBundle\Snappy\Response\PdfResponse;
use Knp\Snappy\Pdf;
use function Safe\sprintf;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Session\Flash\FlashBagInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Security\Core\Authorization\AuthorizationCheckerInterface;
use Twig\Environment;

final class DownloadVoucherPdfAction
{
    /** @var VoucherRepositoryInterface */
    private $voucherRepository;

    /** @var AuthorizationCheckerInterface */
    private $authChecker;

    /** @var FlashBagInterface */
    private $flashBag;

    /** @var VoucherChannelConfigurationProviderInterface */
    private $configurationProvider;

    /** @var Environment */
    private $twig;

    /** @var Pdf */
    private $snappy;

    public function __construct(
        VoucherRepositoryInterface $voucherRepository,
        AuthorizationCheckerInterface $authChecker,
        FlashBagInterface $flashBag,
        VoucherChannelConfigurationProviderInterface $configurationProvider,
        Environment $twig,
        Pdf $snappy
    ) {
        $this->voucherRepository = $voucherRepository;
        $this->authChecker = $authChecker;
        $this->flashBag = $flashBag;
        $this->configurationProvider = $configurationProvider;
        $this->twig = $twig;
        $this->snappy = $snappy;
    }

    public function __invoke(Request $request, int $id): Response
    {
        $voucher = $this->voucherRepository->find($id);
        if (!$voucher instanceof VoucherInterface) {
            throw new NotFoundHttpException('Gift card not found');
        }
        if (!$this->authChecker->isGranted([VoucherVoter::READ], $voucher)) {
            $this->flashBag->add('error', 'dedi_sylius_voucher.voucher.read_error');

            /** @var string $redirectUrl */
            $redirectUrl = filter_var($request->headers->get('referer'), \FILTER_SANITIZE_URL);

            return new RedirectResponse($redirectUrl);
        }

        $configuration = $this->configurationProvider->getConfigurationForVoucher($voucher);
        if (!$configuration instanceof VoucherConfigurationInterface) {
            throw new NotFoundHttpException(sprintf('Configuration not found for voucher %d', $voucher->getId()));
        }

        $html = $this->twig->render('@DediSyliusVoucherPlugin/Shop/Voucher/pdf.html.twig', [
            'voucher' => $voucher,
            'configuration' => $configuration,
        ]);

        return new PdfResponse($this->snappy->getOutputFromHtml($html), 'voucher.pdf');
    }
}
